from base_st_classes import *

## class OpenSuite(NFSSuite):
##     """Test operation 18: OPEN

##     FIXME: Verify that this eqv.part is correct, when updated state
##     description is available.

##     FIXME: Add test for file named "." and ".." in
##     open_claim_delegate_cur4.file, open_claim4.file and
##     open_claim4.file_delegate_prev. 

##     Equivalence partitioning:

##     Input Condition: seqid
##         Valid equivalence classes:
##             correct seqid(10)
##         Invalid equivalence classes:
##             too small seqid(11)
##             too large seqid(12)
##     Input Condition: share_access
##         Valid equivalence classes:
##             valid share_access(20)
##         Invalid equivalence classes:
##             invalid share_access(21)
##     Input Condition: share_deny
##         Valid equivalence classes:
##             valid share_deny(30)
##         Invalid equivalence classes:
##             invalid share_deny(31)
##     Input Condition: owner.clientid
##         Valid equivalence classes:
##             valid clientid(40)
##         Invalid equivalence classes:
##             stale clientid(41)
##     Input Condition: owner.opaque
##         Valid equivalence classes:
##             valid owner(50)
##         Invalid equivalence classes:
##             invalid owner(51)
##     Input Condition: openhow.opentype
##         Valid equivalence classes:
##             OPEN_NOCREATE4(60)
##             OPEN_CREATE4(61)
##         Invalid equivalence classes:
##             invalid openhow.opentype(62)
##     Input Condition: openhow.how
##         Valid equivalence classes:
##             UNCHECKED4(70)
##             GUARDED4(71)
##             EXCLUSIVE4(72)
##         Invalid equivalence classes:
##             invalid openhow.how(73)
##     Input Condition: openhow.how.createattrs
##         Valid equivalence classes:
##             valid createattrs(80)
##         Invalid equivalence classes:
##             invalid createattrs(80)
##     Input Condition: openhow.how.createverf
##         Valid equivalence classes:
##             matching verifier(90):
##             non-matching verifier(91)
##         Invalid equivalence classes:
##             -
##     Input Condition: claim.claim
##         Valid equivalence classes:
##             CLAIM_NULL(100)
##             CLAIM_PREVIOUS(101)
##             CLAIM_DELEGATE_CUR(102)
##             CLAIM_DELEGATE_PREV(103)
##         Invalid equivalence classes:
##             invalid claim.claim(104)
##     Input Condition: claim.file:
##         Valid equivalence classes:
##             valid filename(110)
##         Invalid equivalence classes:
##             non-utf8 filename(111)
##     Input Condition: claim.delegate_type
##         Valid equivalence classes:
##             valid claim.delegate_type(120)
##         Invalid equivalence classes:
##             invalid claim.delegate_type(121)
##     Input Condition: claim.delegate_cur_info.delegate_stateid
##         Valid equivalence classes:
##             valid stateid(130)
##         Invalid equivalence classes:
##             invalid stateid(131)
##     Input Condition: claim.delegate_cur_info.file
##         Valid equivalence classes:
##             valid filename(140)
##         Invalid equivalence classes:
##             invalid filenname(141)
##     Input Condition: claim.file_delegate_prev
##         Valid equivalence classes:
##             valid filename(150)
##         Invalid equivalence classes:
##             invalid filename(151)
##     """

class OpenSuite(NFSSuite):
    """Test operation 18: OPEN
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Tests that should succeed
    #
    def testCreateUncheckedFile(self):
        """OPEN normal file with create and unchecked flags
        """

        self.init_connection()
        
        # Create the file
        attrs = nfs4lib.dict2fattr({ FATTR4_MODE: 0644, FATTR4_SIZE: 32 })
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        # Create the file again...it should ignore attrs
        attrs = nfs4lib.dict2fattr({ FATTR4_MODE: 0600, FATTR4_SIZE: 16 })
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        # Ensure mode unchanged 
        ops = [self.ncl.putfh_op(self.fh)]
        ops.append(self.ncl.getattr([FATTR4_MODE, FATTR4_SIZE]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        obj = res.resarray[-1].arm.arm.obj_attributes
        d = nfs4lib.fattr2dict(obj)
        self.assertEqual(d[FATTR4_MODE], 0644, "Mode was changed")
        self.assertEqual(d[FATTR4_SIZE], 32, "Size was changed")
        
        # Create the file again, it should truncate size to 0 and ignore other attrs
        attrs = nfs4lib.dict2fattr({ FATTR4_MODE: 0600, FATTR4_SIZE: 0 })
        self.fh, self.stateid = self.ncl.create_confirm(attrs)
        
        # Ensure mode unchanged except for size 
        ops = [self.ncl.putfh_op(self.fh)]
        ops.append(self.ncl.getattr([FATTR4_MODE, FATTR4_SIZE]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        obj = res.resarray[-1].arm.arm.obj_attributes
        d = nfs4lib.fattr2dict(obj)
        self.assertEqual(d[FATTR4_MODE], 0644, "Mode was changed")
        self.assertEqual(d[FATTR4_SIZE], 0, "Size was not truncated")

    def testCreatGuardedFile(self):
        """OPEN normal file with create and guarded flags
        """
        self.init_connection()
        
        # Create the file
        self.fh, self.stateid = self.ncl.create_confirm(mode=GUARDED4)
        
        # Create the file again, should return an error
        seqid = self.ncl.seqid
        self.ncl.create_confirm(mode=GUARDED4, error=[NFS4ERR_EXIST])
        self.ncl.seqid = seqid
       
    def testCreatExclusiveFile(self):
        """OPEN normal file with create and exclusive flags
        """
        self.init_connection()
        
        # Create the file
        self.fh, self.stateid = self.ncl.create_confirm(mode=EXCLUSIVE4,
                                                        verifier=0x12345678)

        # Create the file again, should return an error
        seqid = self.ncl.seqid
        self.ncl.create_confirm(mode=EXCLUSIVE4, verifier=0x87654321L,
                            error=[NFS4ERR_EXIST])
        self.ncl.seqid = seqid
        
    def testOpenFile(self):
        """OPEN normal file with nocreate flag
        """
        self.init_connection()
        dir = self.regfile[0:-1]
        name = self.regfile[-1]
        self.fh, self.stateid = self.ncl.open_confirm(dir,name)

    #
    # Testing various errors
    #
    def testOpenVaporFile(self):
        """OPEN non-existant file with nocreate flag should return NFS4ERR_NOENT
        """
        self.init_connection()
        dir = self.vaporfile[0:-1]
        name = self.vaporfile[-1]
        self.ncl.open_confirm(dir,name,error=[NFS4ERR_NOENT])

    def testDir(self):
        """OPEN with a directory should return NFS4ERR_ISDIR
        """
        self.init_connection()
        dir = self.dirfile[0:-1]
        name = self.dirfile[-1]
        self.ncl.open_confirm(dir,name,error=[NFS4ERR_ISDIR])

    def testLink(self):
        """OPEN with a symlink should return NFS4ERR_SYMLINK
        """
        self.init_connection()
        dir = self.linkfile[0:-1]
        name = self.linkfile[-1]
        self.ncl.open_confirm(dir,name,error=[NFS4ERR_SYMLINK])

    def testNonfile(self):
        """OPEN with various nonfile objects should return NFS4ERR_INVAL
        """
        self.init_connection()

        for pathcomps in [self.blockfile,
                          self.charfile,
                          self.socketfile,
                          self.fifofile]:
            dir = pathcomps[0:-1]
            name = pathcomps[-1]
            self.ncl.open_confirm(dir,name,error=[NFS4ERR_INVAL])

    def testNoCfh(self):
        """OPEN with no cfh set should return NFS4ERR_NOFILEHANDLE
        """
        self.init_connection()
        ops = [self.ncl.open(self.obj_name)]
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_NOFILEHANDLE])

    def testZeroLenName(self):
        """OPEN with zero length name should return NFS4ERR_INVAL"""
        self.init_connection()
        self.fh, self.stateid = self.ncl.create_confirm(name="", error=[NFS4ERR_INVAL])

    def testLongName(self):
        """OPEN should fail with NFS4ERR_NAMETOOLONG with long filenames"""
        self.init_connection()
        name = "a"*512
        self.fh, self.stateid = self.ncl.create_confirm(name=name, error=[NFS4ERR_NAMETOOLONG])

    def testNotDir(self):
        """OPEN with cfh not a directory should return NFS4ERR_NOTDIR
        """
        self.init_connection()
        dir = self.regfile
        self.ncl.open_confirm(dir,error=[NFS4ERR_NOTDIR])
       
    def testInvalidUtf8(self):
        """OPEN with bad UTF-8 name strings should fail
        """
        self.init_connection()
        badnames = self.get_invalid_utf8strings()

        for name in badnames:
            self.fh, self.stateid = self.ncl.create_confirm(name=name, error=[NFS4ERR_INVAL])

    def testInvalidAttrmask(self):
        """OPEN should fail with NFS4ERR_INVAL on invalid attrmask

        Comments: We are using a read-only attribute on OPEN, which
        should return NFS4ERR_INVAL. 
        """
        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_LINK_SUPPORT: TRUE})
        self.fh, self.stateid = self.ncl.create_confirm(attrs, error=[NFS4ERR_INVAL])

    def testUnsupportedAttributes(self):
        """OPEN should fail with NFS4ERR_ATTRNOTSUPP on unsupported attrs
        """
        self.init_connection()
        supported = self.supportedAttrs(self.dirfile)
        attrbitnum_dict = nfs4lib.get_attrbitnum_dict()
        for (attr,value) in self.ncl.writable_attrs:
            bitnum = attrbitnum_dict[attr]
            if not (supported & (2**bitnum)):
                attrs = nfs4lib.dict2fattr({bitnum: value})
                self.fh, self.stateid = self.ncl.create_confirm(attrs, error=[NFS4ERR_ATTRNOTSUPP])

    def testClaimPrev(self):
        """OPEN with CLAIM_PREVIOUS should return NFS4ERR_NO_GRACE

        Note this assumes test is run after grace period has expired.
        """
        self.init_connection()
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.open_claim_prev(self.fh, error=[NFS4ERR_NO_GRACE])

    def xxxtestDeleg(self):
        """OPEN with a delegation claim"""

        # self.init_connection clone
        PROG_NUM = 0x40483719
        id = self.ncl.gen_uniq_id()
        verifier = self.ncl.gen_random_64()
        setclientidop = self.ncl.setclientid(verifier,id,PROG_NUM,'tcp',
                                        #self.ncl.sock.getsockname()[0]+'.176.84')
                                        self.ncl.sock.getsockname()[0]+'.176.84')
        res = self.ncl.do_ops([setclientidop])
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        self.ncl.clientid = clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        setclientid_confirmop = self.ncl.setclientid_confirm_op(clientid,idconfirm)
        res = self.ncl.do_ops([setclientid_confirmop])
        self.assert_OK(res)

        # Create a file
        self.fh, self.stateid = self.ncl.create_confirm()

