from base_st_classes import *

class WriteSuite(NFSSuite):
    """Test operation 38: WRITE

##     FIXME: Complete. 
##     FIXME: Write to named attribute. 

##     Equivalence partitioning:

##     Input Condition: current filehandle
##         Valid equivalence classes:
##             file(10)
##             named attribute(11)
##         Invalid equivalence classes:
##             dir(12)
##             special device files(13)
##             invalid filehandle(14)
##     Input Condition: stateid
##         Valid equivalence classes:
##             all bits zero(20)
##             all bits one(21)
##             valid stateid from open(22)
##         Invalid equivalence classes:
##             invalid stateid(23)
##     Input Condition: offset
##         Valid equivalence classes:
##             zero(30)
##             nonzero(31)
##         Invalid equivalence classes:
##             -
##     Input Condition: stable
##         Valid equivalence classes:
##             UNSTABLE4(40)
##             DATA_SYNC4(41)
##             FILE_SYNC4(42)
##         Invalid equivalence classes:
##             invalid constant(43)
##     Input Condition: data
##         Valid equivalence classes:
##             no data(50)
##             some data(51)
##         Invalid equivalence classes:
##             -
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Testcases covering valid equivalence classes.
    #
    def testSimpleWrite(self):
        """WRITE with stateid=zeros and UNSTABLE4

        Covered valid equivalence classes: 10, 20, 30, 40, 51
        """

        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        self.ncl.write_file(stateid=stateid4(0, ""), how=UNSTABLE4)
        eof, data = self.ncl.read_file(0,10)
        if data != "data" + "\0"*6:
            self.fail("READ does not correspond to previous WRITE")

    def testStateidOne(self):
        """WRITE with stateid=ones and DATA_SYNC4

        Covered valid equivalence classes: 10, 21, 31, 41, 51
        """

        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        stateval = nfs4lib.long2opaque(0xffffffffffffffffffffffffL)
        stateid = stateid4(0xffffffffL, stateval)
        count, how = self.ncl.write_file(10, "data", stateid, DATA_SYNC4)
        if how == UNSTABLE4:
            self.fail("Asked for DATA_SYNC4, got UNSTABLE4")
        eof, data = self.ncl.read_file(0,15)
        if data != "\0"*10 + "data\0":
            self.fail("READ does not correspond to previous WRITE")
        
    def testWithOpen(self):
        """WRITE with open and FILE_SYNC4

        Covered valid equivalence classes: 10, 22, 31, 42, 51
        """

        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        count, how = self.ncl.write_file(50, stateid=self.stateid)
        if how != FILE_SYNC4:
            self.fail("Asked for FILE_SYNC4, did not get it")
        eof, data = self.ncl.read_file(0,100)
        if len(data) != 54 or data[-5:] != "\0data" or not eof:
            self.fail("READ does not correspond to previous WRITE")

    def testNoData(self):
        """WRITE with no data"""

        # Open file
        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        # Get time_modify
        ops = [self.ncl.putfh_op(self.fh)]
        ops.append(self.ncl.getattr([FATTR4_TIME_MODIFY]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        time_prior = res.resarray[-1].arm.arm.obj_attributes
        time_prior = nfs4lib.fattr2dict(time_prior)[FATTR4_TIME_MODIFY]
        
        time.sleep(1)
        
        count, how = self.ncl.write_file(5, "", stateid=self.stateid)
        if count:
            self.fail("WRITE with no data returned count=%d" % count)

        # No ensure time_modify was unchanged
        ops = [self.ncl.putfh_op(self.fh)]
        ops.append(self.ncl.getattr([FATTR4_TIME_MODIFY]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        time_after = res.resarray[-1].arm.arm.obj_attributes
        time_after = nfs4lib.fattr2dict(time_after)[FATTR4_TIME_MODIFY]

        if self.compareTimes(time_prior,time_after) != 0:
            self.fail("WRITE with no data affected time_modify")

    def testLargeData(self):
        """WRITE with a large amount of data"""
        # FRED note ethereal complains about all sorts of
        # unreassembled packets with truncated data, but this seems OK
        
        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        data = "abcdefghijklmnopq" * 0x10000
        self.ncl.write_file(0, data[:0x8000], self.stateid)
        self.ncl.write_file(0x8000, data[0x8000:0x10000], self.stateid)

        eof, readdata = self.ncl.read_file(0,0x100000,self.stateid)
        if readdata[:0x8000] != data[:0x8000]:
            self.fail("READ does not correspond to previous WRITE")
        
        eof, readdata = self.ncl.read_file(0x8000,0x100000,self.stateid)
        if readdata[:0x8000] != data[0x8000:0x10000]:
            self.fail("READ does not correspond to previous WRITE")
        
    #
    # Testcases covering invalid equivalence classes.
    #
## ##     def xtestInvalid(self):
## ##         pass
    def testDirFh(self):
        """WRITE with (cfh)=directory should return NFS4ERR_ISDIR

        Covered invalid equivalence classes: 12
        """
        self.ncl.write_file(path=self.dirfile, error=[NFS4ERR_ISDIR])

    def testSpecials(self):
        """WRITE with (cfh)=device files should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 13
        """
        for path in [self.blockfile,
                     self.charfile,
                     self.linkfile,
                     self.socketfile,
                     self.fifofile]:
            self.ncl.write_file(path=path, error=[NFS4ERR_INVAL])

    def testNoFh(self):
        """WRITE without (cfh) should return NFS4ERR_NOFILEHANDLE
 
        Covered invalid equivalence classes: 14
        """
        stateid = stateid4(0, "")
        writeop = self.ncl.write("", stateid)
        res = self.ncl.do_ops([writeop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])
 
    def testBadStateid(self):
        """WRITE with bad stateid should return NFS4ERR_BAD_STATEID

        Covered invalid equivalence classes: 23

        NOTE Mangling functions look into server opaque data, thus are
        specific to the CITI linux server
        """
        # Get a valid stateid
        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        # Mangle it
        badid = self.makeBadID(self.stateid)

        self.ncl.write_file(0, "", badid, path=self.regfile,
                            error=[NFS4ERR_BAD_STATEID])

    def testStaleStateid(self):
        """WRITE with stale stateid should return NFS4ERR_STALE_STATEID

        Covered invalid equivalence classes: 23

        NOTE Mangling functions look into server opaque data, thus are
        specific to the CITI linux server
        """
        # Get a valid stateid
        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        # Mangle it
        staleid = self.makeStaleID(self.stateid)

        self.ncl.write_file(0, "", staleid, path=self.regfile,
                            error=[NFS4ERR_STALE_STATEID])

    def testOpenMode(self):
        """WRITE with file opened in READ mode should return NFS4ERR_OPENMODE

        """

        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 32, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs, access=OPEN4_SHARE_ACCESS_READ)

        self.ncl.write_file(stateid=self.stateid, error=[NFS4ERR_OPENMODE])
